import { Target, CONSTANTS } from '../types';

export const generateTargets = () => {
	const newTargets: Target[] = [];
	const PADDING = CONSTANTS.TARGET_MAX_RADIUS * 2;
  
	// Get browser window aspect ratio
	const windowWidth = window.innerWidth;
	const windowHeight = window.innerHeight;
	const aspectRatio = windowWidth / windowHeight;
	
	// Calculate grid dimensions based on aspect ratio
	const gridWidth = CONSTANTS.VIRTUAL_CANVAS_SIZE;
	const gridHeight = gridWidth / aspectRatio;
	
	// Calculate cells based on aspect ratio
	const totalCells = CONSTANTS.TARGET_COUNT;
	const numColumns = Math.ceil(Math.sqrt(totalCells * aspectRatio));
	const numRows = Math.ceil(totalCells / numColumns);
	
	const cellWidth = (gridWidth - PADDING * 2) / numColumns;
	const cellHeight = (gridHeight - PADDING * 2) / numRows;
  
	for (let i = 0; i < CONSTANTS.TARGET_COUNT; i++) {
		let validPosition = false;
		let newTarget: Target;

		// Calculate the grid position
		const column = i % numColumns;
		const row = Math.floor(i / numColumns);
    
		while (!validPosition) {
			const radius = CONSTANTS.TARGET_MIN_RADIUS + 
				Math.random() * (CONSTANTS.TARGET_MAX_RADIUS - CONSTANTS.TARGET_MIN_RADIUS);
        
			// Base position from grid
			const baseX = PADDING + column * cellWidth;
			const baseY = PADDING + row * cellHeight;
        
			// Add some randomness within the cell
			const x = baseX + Math.random() * (cellWidth - radius * 2);
			const y = baseY + Math.random() * (cellHeight - radius * 2);
        
			newTarget = { x, y, radius };
			validPosition = true;

			// Check for overlaps
			for (const existing of newTargets) {
				const distance = Math.hypot(existing.x - x, existing.y - y);
				if (distance < (existing.radius + radius) * 1.5) {
					validPosition = false;
					break;
				}
			}
		}
		newTargets.push(newTarget!);
	}
	return newTargets;
};

export const getRandomTarget = (currentTarget: number | null, maxTargets: number): number => {
	let newTarget: number;
	do {
		newTarget = Math.floor(Math.random() * maxTargets);
	} while (newTarget === currentTarget);
	return newTarget;
};
